/**
 * Fichero : Grafo.c      Fecha: 08/03/2019
 * Autores: 
 *
 *  Fichero con funciones para implementar el TAD Grafo y 
 *  funciones relacionadas.
 *
 * (C) 2016-2019 Escuela Politecnica Superior, Universidad Autonoma de Madrid
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "grafo.h"


/*
 *  Funcion para crear un Grafo vacio con n_vertices 
 */
Grafo *crearGrafo(int n_vertices)
{
  Grafo *g;
  int i;

  g = malloc(sizeof(Grafo));
  g->n_vertices = n_vertices;
  g->n_aristas = 0;
  g->es_dirigido = 0;

  g->vertices = malloc(sizeof(Vertice)*g->n_vertices);

  for(i = 0; i < g->n_vertices; i++) {
      g->vertices[i].id = i;
      g->vertices[i].aristas = NULL;
  }

  return g;
}
/*
 *  Funcion para crear un Grafo dirigido vacio con n_vertices. Para los 
 *  grafos dirigidos, cada vez que se aniada una arista nueva al grafo se
 *  incluira tanto en la lista de adyacencia del vertice origen y destino 
 */
Grafo *crearGrafoDirigido(int n_vertices)
{
  Grafo *g;

  g = crearGrafo(n_vertices);
  g->es_dirigido = 1;

  return g;
}

/*
 * Funcion interna al TAD, no se declara en el .h ya que es util solo
 * internamente. Añade una arista sin mirar si el grafo debe ser dirigido o no
 *
 * Esta función es llamada solo desde aniadirArista, que es la que se encarga de
 * comprobar si el grafo es dirigido o no.
 */
void aniadirAristaUtil(Grafo *g, int orig, int dest, int peso)
{
  Arista *a;

  assert(orig>=0 && orig<g->n_vertices);
  assert(dest>=0 && dest<g->n_vertices);

  /*Inicializa la rama*/
  a = malloc(sizeof(Arista));
  if (!a) return;
  a->dest = dest;
  a->peso = peso;

  /*Inserta la rama al inicio de la lista enlazada del vertice*/
  a->sig = g->vertices[orig].aristas;
  g->vertices[orig].aristas = a;

  g->n_aristas++;
}
/*
 * Aniade una arista al Grafo. Si el grafo es dirigido se incluirá la arista
 * como orig->dest y como dest->orig
 */
void aniadirArista(Grafo *g, int orig, int dest, int peso)
{
  aniadirAristaUtil(g, orig, dest, peso);
  if (!esDirigido(g)) aniadirAristaUtil(g, dest, orig, peso);
}

/*
 *  Libera todo la memoria reservada para el Grafo 
 */
void borrarGrafo(Grafo *g)
{
    int i;

    for(i = 0; i< g->n_vertices; i++) {
        Arista *aux, *mas;

        mas = g->vertices[i].aristas;
        while(mas) {
            aux = mas->sig; 
            free(mas);
            mas = aux;
        };
    }

    free(g->vertices);
    free(g);
}

/*
 *  Devuelve si el grafo g es dirigido
 */
int esDirigido(Grafo *g)
{
  return g->es_dirigido;
}

/*
 *  Imprimeel Grafo 'g'
 */
void imprimirGrafo(Grafo *g)
{
  int i;

  printf("%d\n", g->n_vertices);
  printf("%d\n", g->n_aristas);

  for(i = 0; i < g->n_vertices; i++) {
    Arista *mas;

    mas = g->vertices[i].aristas;
    while(mas) {
      printf("%d %d %d\n", i, mas->dest, mas->peso);
      mas = mas->sig; 
    }
  }
}

/*
 * Implementar una función que realice un recorrido en anchura. Esta función
 * recibe como argumento un grafo y debe imprimir las aristas en el orden dado
 * por un recorrido en anchura. 
 */
void imprimirEnAnchura(Grafo *g, int partida)
{
}

/*
 * Implementar una función que realice un recorrido en profundidad. Esta función
 * recibe como argumento un grafo y debe imprimir las aristas en el orden dado
 * por un recorrido en profundidad.
 */
void imprimirEnProfundidad(Grafo *g, int partida)
{
}

/* Función que cuente el número de arcos de entrada y de salida para cada
 * vértice del grafo g.
 *
 * Salida:
 *   Array de de dimensión 2*n_vertices que contiene en la posición 2i y 2i+1 el
 *   número de arcos de salida y de entrada respectivamente para el vértice i,
 *   con i = 1...n_vertices
 *
 */
int *contarGradosVertices(Grafo *g)
{
}

/*
 * Función que realiza el recorrido en orden topológico del grafo g.
 *
 * Salida:
 *   Si es posible hacer el recorrido completo del grafo, la función debe
 *   devolver 1 y si no 0.
 */
int imprimirOrdenacionTopologica(Grafo *g)
{
}

/*
 * Funcion que genera un Grafo conexo regular con 2*n_vertices=n_aristas
 *   en el que cada vertice esta unido a sus 4 vecinos mas proximos. La 
 *   poderacion de las aristas es aleatoria y va de 1 a 100.
 *
 */
Grafo *crearGrafoRegular(int n_vertices)
{
  int i;
  Grafo *g;

  if (n_vertices<5) return NULL;

  g = crearGrafoDirigido(n_vertices);

  for(i = 0; i < n_vertices; i++) {
    aniadirArista(g, i, (i+1)%n_vertices, 1+(int) (100.0*rand()/(RAND_MAX+1.0)));
    aniadirArista(g, i, (i+2)%n_vertices, 1+(int) (100.0*rand()/(RAND_MAX+1.0)));
  }

  return g;
}

/**
 *  Devuelve un número aleat_entreorio en el rango [max, min]
 */
int aleat_entre(int min, int max)
{
    double rango = max - min + 1;
    return min + (int)(rango*rand()/(RAND_MAX+1.0));
}

/*
 * Genera un Grafo dirigido aleatorio de n_vertices y n_aristas  con una  
 *   ponderacion de las aristas entre min_pond y max_pond
 */
Grafo *crearGrafoAleatorio(int n_vertices, int n_aristas, int min_pond, int max_pond)
{
  int n_aristas_max, i, *aristas;
  Grafo *g;

  /*Comprobamos que los parametros de entrada son consistentes*/
  if (n_aristas > n_vertices*(n_vertices-1) || n_vertices<0 || n_aristas<0)
  {
      return 0;
  }

  /*Reservamos memoria para:*/
  /* calcular las posibles aristas */
  n_aristas_max = n_vertices*n_vertices;
  aristas = malloc(sizeof(int) * n_aristas_max);
  if (aristas == NULL ) return 0;

  for (i = 0; i < n_aristas_max; i++) {
    aristas[i] = i;
  }

  g = crearGrafoDirigido(n_vertices);

  for (i = 0; i < n_aristas; i++) {
    int ir = aleat_entre(i, n_aristas_max-1);
    int aux     = aristas[ir];
    aristas[ir] = aristas[i];
    aristas[i] = aux;
    if (aristas[i]/n_vertices == aristas[i]%n_vertices) {
      i--;
    }
    else {
      aniadirArista(g, aristas[i]/n_vertices, aristas[i]%n_vertices, 
                                         1+(int) (100.0*rand()/(RAND_MAX+1.0)));
    }
  }

  /*Liberamos memoria*/
  free(aristas);

  return g;
}

